body {
    background-color: white;
    border: 0;
    box-sizing: border-box;
    -moz-box-sizing: border-box;
    color: black;
    margin: 0;
    overflow-x: hidden;
    padding: 0;
    width: 100%;
    }
.fa-icon {
    cursor: pointer;
    font-size: 150%;
    padding: 0.4em 0.6em;
    }
.fa-icon:hover {
    background-color: #eee;
    }
#toolbar {
    background-color: white;
    border: 0;
    border-bottom: 1px solid #ccc;
    box-sizing: border-box;
    -moz-box-sizing: border-box;
    left: 0;
    margin: 0;
    padding: 0.5em 1em;
    position: fixed;
    top: 0;
    width: 100%;
    z-index: 10;
    }
#toolbar > div {
    display: flex;
    padding: 0.5em;
    white-space: nowrap;
    }
#pageSelector {
    width: 28em;
    margin-right: 0.5em;
    padding: 0.2em 0;
    }

@keyframes popupPanelShow {
    from { opacity: 0; }
    to { opacity: 1; }
}
#popupPanelContainer {
    background: white;
    border: 1px solid gray;
    display: none;
    overflow: hidden;
    position: fixed;
    right: 0;
    z-index: 2000;
    }
body.popupPanelOn #popupPanelContainer {
    animation-duration: 0.25s;
    animation-name: popupPanelShow;
    display: block;
    }
#popupPanelContainer.hide {
    width: 6em !important;
}
#popupPanelContainer > iframe {
    border: 0;
    padding: 0;
    margin: 0;
    overflow: hidden;
    width: 100%;
    }
#popupPanelContainer.hide > iframe {
    display: none;
    }

#popupPanelButton use {
    transform: scale(1, 0.4);
    }
body.popupPanelOn #popupPanelButton use {
    transform: scale(1, 1);
    }
body.compactView #compactViewToggler use {
    transform: scale(1, -1);
    transform-origin: center;
    }
#filterButton {
    opacity: 0.25;
    }
body.f #filterButton {
    opacity: 1;
    }
#filterInput.bad {
    background-color: #fee;
    }
#maxEntries {
    margin: 0 2em;
    }
input:focus {
    background-color: #ffe;
    }
#content {
    font-family: "Segoe UI", "Roboto", "Oxygen", "Ubuntu", "Cantarell", "Fira Sans", "Droid Sans", "Helvetica Neue", sans-serif;
    font-size: 13px;
    width: 100%;
    }

#content table {
    border: 0;
    border-collapse: collapse;
    direction: ltr;
    table-layout: fixed;
    width: 100%;
    }
#content table > colgroup > col:nth-of-type(1) {
    width: 4.6em;
    }
#content table > colgroup > col:nth-of-type(2) {
    width: 25%;
    }
#content table > colgroup > col:nth-of-type(3) {
    width: 2.2em;
    }
#content table > colgroup > col:nth-of-type(4) {
    width: 5.4em;
    }
#content table > colgroup > col:nth-of-type(5) {
    width: calc(100% - 4.6em - 25% - 2.2em - 5.4em - 1.8em);
    }
#content table > colgroup > col:nth-of-type(6) {
    width: 1.8em;
    }
#content table tr {
    background-color: #fafafa;
    }
body.f table tr.f {
    display: none;
    }
#content table tr:nth-of-type(2n+1) {
    background-color: #eee;
    }

#content table tr.cat_info {
    color: #00f;
    }
#content table tr.blocked {
    color: #f00;
    }
#content table tr.doc {
    background-color: #666;
    color: white;
    text-align: center;
    }

body #content td {
    border: 1px solid #ccc;
    min-width: 0.5em;
    padding: 3px;
    vertical-align: top;
    white-space: normal;
    word-break: break-all;
    word-wrap: break-word;
    }
#content table tr td:first-of-type {
    border-left: none;
    }
#content table tr td:last-of-type {
    border-right: none;
    }
body.compactView #content tr:not(.vExpanded) td {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
    }

#content table tr td:nth-of-type(1) {
    cursor: default;
    text-align: right;
    white-space: nowrap;
    }
#content table tr td:nth-of-type(2):not([colspan]) {
    direction: rtl;
    text-align: right;
    unicode-bidi: plaintext;
    }
#content table tr[data-tabid="-1"] td:nth-of-type(2):not([colspan]) {
    position: relative;
    }
#content table tr td:nth-of-type(3) {
    position: relative;
    }
/* visual for tabless network requests */
#content table tr[data-tabid="-1"] td:nth-of-type(3)::before {
    border: 5px solid #bbb;
    border-bottom: 0;
    border-top: 0;
    bottom: 0;
    content: '\00a0';
    left: 0;
    position: absolute;
    right: 0;
    top: 0;
    width: calc(100% - 10px);
    }
#content table tr.tab:not(.canMtx) {
    opacity: 0.3;
    }
#content table tr.tab:not(.canMtx):hover {
    opacity: 0.7;
    }
#content table tr.cat_net td:nth-of-type(3) {
    cursor: pointer;
    font: 12px monospace;
    text-align: center;
    white-space: nowrap;
    }
#content table tr.cat_net td:nth-of-type(5) {
    }
#content table tr.cat_net td:nth-of-type(5) > span > * {
    opacity: 0.6;
    }
#content table tr.cat_net td:nth-of-type(5) > span > b:first-of-type {
    opacity: 1;
    }

.modalDialog {
    align-items: center;
    background-color: rgba(0, 0, 0, 0.5);
    display: flex;
    height: 100vh;
    justify-content: center;
    left: 0;
    position: fixed;
    top: 0;
    width: 100vw;
    z-index: 5000;
    }
.modalDialog > .dialog {
    background-color: white;
    font: 15px httpsb,sans-serif;
    min-width: fit-content;
    padding: 0.5em;
    width: 90%;
    }

#ruleEditor section {
    display: flex;
    }
.scopeWidget {
    line-height: 2.5em;
    margin-bottom: 0.5em;
    }
#specificScope, .ruleCell:nth-of-type(1) {
    flex-grow: 1;
    }
#globalScope, .ruleCell:nth-of-type(2) {
    width: 4em;
    }
.ruleEditorToolbar {
    display: flex;
    flex-direction: column;
    justify-content: space-around;
    margin-left: 0.5em;
    padding: 0.2em;
    }
.ruleEditorToolbar .fa-icon {
    padding: 0.4em;
    }

.fa-icon.scopeRel {
    color: #24c;
    fill: #24c;
    }
body[data-scope="*"] .fa-icon.scopeRel {
    color: #000;
    fill: #000;
    }
.ruleWidgets {
    display: flex;
    flex-direction: column;
    flex-grow: 1;
    }
.ruleRow {
    display: flex;
    line-height: 2em;
    margin-top: 1px;
    }
.ruleCell {
    background-color: #eee;
    border: 1px dotted rgba(0,0,0,0.2);
    cursor: pointer;
    display: inline-block;
    margin-left: 1px;
    padding: 1px;
    position: relative;
    }
.ruleCell:hover {
    border-style: solid;
    }
.ruleCell:nth-of-type(1) {
    margin-left: 0;
    text-align: right;
    }
.ruleCell:nth-of-type(2) {
    text-align: center;
    }
.ruleCell[data-tcolor="1"] {
    border-color: #debaba;
    color: black;
    background-color: #f8d0d0;
    }
#ruleEditor.colorblind .ruleCell[data-tcolor="1"] {
    border-color: rgba(0, 19, 110, 0.3);
    color: black;
    background-color: rgba(0, 19, 110, 0.2);
    }
.ruleCell[data-tcolor="2"] {
    border-color: #bad6ba;
    color: black;
    background-color: #d0f0d0;
    }
#ruleEditor.colorblind .ruleCell[data-tcolor="2"] {
    border-color: rgba(255, 194, 57, 0.3);
    color: black;
    background-color: rgba(255, 194, 57, 0.2);
    }
.ruleCell[data-tcolor="129"] {
    color: white;
    background-color: #c00;
    }
#ruleEditor.colorblind .ruleCell[data-tcolor="129"] {
    color: white;
    background-color: rgb(0, 19, 110);
    }
.ruleCell[data-tcolor="130"] {
    color: white;
    background-color: #080;
    }
#ruleEditor.colorblind .ruleCell[data-tcolor="130"] {
    border-color: rgb(255, 194, 57);
    color: black;
    background-color: rgb(255, 194, 57);
    }
.ruleCell[data-pcolor="129"] {
    background-image: url('../img/permanent-black-small.png');
    background-repeat: no-repeat;
    background-position: -1px -1px;
    }
#ruleEditor.colorblind .ruleCell[data-pcolor="129"] {
    background-image: url('../img/permanent-black-small-cb.png');
    }
.ruleCell[data-pcolor="130"] {
    background-image: url('../img/permanent-white-small.png');
    background-repeat: no-repeat;
    background-position: -1px -1px;
    }
#ruleEditor.colorblind .ruleCell[data-pcolor="130"] {
    background-image: url('../img/permanent-white-small-cb.png');
    }

#ruleActionPicker {
    border: 0;
    height: 100%;
    left: 0;
    margin: 0;
    padding: 0;
    position: absolute;
    top: 0;
    width: 100%;
    z-index: 10;
    }
.allowRule, .blockRule {
    margin: 0;
    border: 0;
    padding: 0;
    position: absolute;
    left: 0;
    width: 100%;
    height: 50%;
    background: transparent;
    }
.allowRule {
    top: 0;
    }
.blockRule {
    top: 50%;
    }
.ruleCell[data-tcolor="1"] .allowRule:hover {
    background-color: #080;
    opacity: 0.25;
    }
.ruleCell[data-tcolor="1"] .blockRule:hover {
    background-color: #c00;
    opacity: 0.25;
    }
.ruleCell[data-tcolor="2"] .allowRule:hover {
    background-color: #080;
    opacity: 0.25;
    }
.ruleCell[data-tcolor="2"] .blockRule:hover {
    background-color: #c00;
    opacity: 0.25;
    }
.ruleCell[data-tcolor="129"] .allowRule:hover {
    background-color: transparent;
    }
.ruleCell[data-tcolor="129"] .blockRule:hover {
    background-color: transparent;
    }
.ruleCell[data-pcolor="130"] .allowRule:hover {
    background-color: transparent;
    }
.ruleCell[data-pcolor="130"] .blockRule:hover {
    background-color: transparent;
    }
#ruleEditor.colorblind .ruleCell[data-tcolor="1"] .allowRule:hover,
#ruleEditor.colorblind .ruleCell[data-tcolor="2"] .allowRule:hover {
    background-color: rgb(255, 194, 57);
    opacity: 0.6;
    }
#ruleEditor.colorblind .ruleCell[data-tcolor="1"] .blockRule:hover,
#ruleEditor.colorblind .ruleCell[data-tcolor="2"] .blockRule:hover {
    background-color: rgb(0, 19, 110);
    opacity: 0.4;
    }
