#cloudWidget {
    background: url("../img/cloud.png") hsl(216, 100%, 93%);
    border-radius: 3px;
    margin: 0.5em 0;
    padding: 1em 1em 0 1em;
    position: relative;
    }
#cloudWidget.hide {
    display: none;
    }
#cloudWidget > button {
    font-size: 180%;
    padding: 0 0.25em;
    position: relative;
    }
#cloudWidget > button[disabled] {
    visibility: hidden;
    }
#cloudWidget > button.error {
    color: red;
    }
#cloudPullAndMerge {
    margin: 0 0.25em;
    }
#cloudPullAndMerge > span:nth-of-type(2) {
    font-size: 50%;
    position: absolute;
    right: 0;
    top: 0;
    }
#cloudWidget > span {
    color: gray;
    display: inline-block;
    font-size: 90%;
    margin: 0 1em;
    padding: 0;
    vertical-align: bottom;
    white-space: pre;
    }
#cloudError {
    color: red;
    margin: 0;
    padding: 0.5em 0;
    }
#cloudError > span {
    font-size: x-small;
    }
#cloudWidget > #cloudCog {
    cursor: pointer;
    font-size: 110%;
    margin: 0;
    opacity: 0.5;
    padding: 0.2em;
    position: absolute;
    top: 0;
    }
body[dir="ltr"] #cloudWidget > #cloudCog {
    right: 0;
    }
body[dir="rtl"] #cloudWidget > #cloudCog {
    left: 0;
    }
#cloudWidget > #cloudCog:hover {
    opacity: 1;
    }
#cloudWidget > #cloudOptions {
    align-items: center;
    -webkit-align-items: center;
    background-color: rgba(0, 0, 0, 0.75);
    bottom: 0;
    display: none;
    justify-content: center;
    -webkit-justify-content: center;
    left: 0;
    position: fixed;
    right: 0;
    top: 0;
    z-index: 2000;
    }
#cloudWidget > #cloudOptions.show {
    display: flex;
    display: -webkit-flex;
    }
#cloudWidget > #cloudOptions > div {
    background-color: white;
    border-radius: 3px;
    padding: 1em;
    text-align: center;
    }
