.fa-icon {
    align-items: center;
    background-color: transparent;
    border: 0;
    display: inline-flex;
    justify-content: center;
    margin: 0;
    padding: 0.1em;
    position: relative;
    user-select: none;
    -webkit-user-select: none;
    }
.fa-icon > * {
    pointer-events: none;
    }
.fa-icon.disabled,
.disabled > .fa-icon,
.fa-icon[disabled],
[disabled] > .fa-icon {
    color: #000;
    fill: #000;
    opacity: 0.25;
    stroke: #888;
    pointer-events: none;
    }
.fa-icon > .fa-icon-badge,
.fa-icon.disabled > .fa-icon-badge {
    display: none;
    }
.fa-icon.fa-icon-badged > .fa-icon-badge {
    background-color: rgba(255, 255, 255, 0.7);
    border-radius: 3px;
    bottom: 1px;
    display: inline-block;
    font-family: sans-serif;
    font-size: 40%;
    position: absolute;
    right: 1px;
    }
.fa-icon.disabled > .fa-icon-badge {
    display: none;
    }

.fa-icon > svg {
    height: 1em;
    width: 1em;
    }
.fa-icon > .fa-icon_cloud-download,
.fa-icon > .fa-icon_cloud-upload,
.fa-icon > .fa-icon_eraser {
    width: calc(1em * 1920 / 1792);
    }
.fa-icon > .fa-icon_exclamation-triangle {
    width: calc(1em * 1794 / 1792);
    }
.fa-icon > .fa-icon_list-alt,
.fa-icon > .fa-icon_reply,
.fa-icon > .fa-icon_reply-all,
.fa-icon > .fa-icon_th {
    width: calc(1em * 1792 / 1792);
    }
.fa-icon > .fa-icon_spinner {
    width: calc(1em * 1728 / 1792);
    }
.fa-icon > .fa-icon_download,
.fa-icon > .fa-icon_puzzle-piece,
.fa-icon > .fa-icon_search-minus,
.fa-icon > .fa-icon_search-plus,
.fa-icon > .fa-icon_unlink {
    width: calc(1em * 1664 / 1792);
    }
.fa-icon > .fa-icon_home {
    width: calc(1em * 1612 / 1792);
    }
.fa-icon > .fa-icon_clock,
.fa-icon > .fa-icon_cog,
.fa-icon > .fa-icon_info-circle,
.fa-icon > .fa-icon_power-off,
.fa-icon > .fa-icon_question-circle,
.fa-icon > .fa-icon_refresh {
    width: calc(1em * 1536 / 1792);
    }
.fa-icon > .fa-icon_arrow-left {
    width: calc(1em * 1472 / 1792);
    }
.fa-icon > .fa-icon_filter {
    width: calc(1em * 1410 / 1792);
    }
.fa-icon > .fa-icon_plus,
.fa-icon > .fa-icon_trash-o {
    width: calc(1em * 1408 / 1792);
    }
.fa-icon > .fa-icon_times {
    width: calc(1em * 1188 / 1792);
    }
.fa-icon > .fa-icon_lock,
.fa-icon > .fa-icon_unlock {
    width: calc(1em *  1152 / 1792);
    }
.fa-icon > .fa-icon_angle-up,
.fa-icon > .fa-icon_double-angle-up {
    width: calc(1em *  998 / 1792);
    }
.fa-icon > .fa-icon_ellipsis-v {
    width: calc(1em * 384 / 1792);
    }
