@font-face {
  font-family: 'httpsb';
  font-style: normal;
  font-weight: normal;
  src: local('httpsb'), url(fonts/Roboto_Condensed/RobotoCondensed-Regular.ttf) format('truetype');
}
@font-face {
  font-family: 'httpsb';
  font-style: normal;
  font-weight: 100;
  src: local('httpsb'), url(fonts/Roboto_Condensed/RobotoCondensed-Light.ttf) format('truetype');
}

body {
    font-size: 14px;
    }
body[dir="ltr"] {
    direction: ltr;
    }
body[dir="rtl"] {
    direction: rtl;
    }

/* http://stackoverflow.com/questions/2011142/how-to-change-the-style-of-title-attribute-inside-the-anchor-tag?answertab=votes */
*[data-i18n-tip] {
    position: relative;
    cursor: pointer;
    }
*[data-i18n-tip]::after {
    content: "";
    opacity: 0;
    }
*[data-i18n-tip]:hover::after {
    background-color: #fffffa;
    border: 1px solid gray;
    border-radius: 3px;
    box-shadow: 1px 1px 3px gray;
    color: black;
    content: attr(data-tip);
    font: 12px sans-serif;
    left: 10vw;
    line-height: 140%;
    min-width: 25vw;
    opacity: 1;
    padding: 4px 6px;
    pointer-events: none;
    position: absolute;
    right: 10vw;
    text-align: start;
    top: 110%;
    -webkit-transition: opacity 0.15s 0.5s;
    transition: opacity 0.15s 0.5s;
    white-space: pre-line;
    z-index: 20;
    }
body .tip-anchor-center[data-i18n-tip]:hover::after {
    left: -10vw;
    right: 10vw;
    }
body[dir="ltr"] .tip-anchor-left[data-i18n-tip]:hover::after,
body[dir="rtl"] .tip-anchor-right[data-i18n-tip]:hover::after {
    left: 0;
    right: unset;
    }
body[dir="ltr"] .tip-anchor-right[data-i18n-tip]:hover::after,
body[dir="rtl"] .tip-anchor-left[data-i18n-tip]:hover::after {
    left: unset;
    right: 0;
    }
body.noTooltips *[data-i18n-tip]::after {
    display: none;
    }

button.custom {
    padding: 0.6em 1em;
    border: 1px solid transparent;
    border-color: #ccc #ccc #bbb #bbb;
    border-radius: 3px;
    background-color: hsl(216, 0%, 75%);
    background-image: linear-gradient(#f2f2f2, #dddddd);
    background-repeat: repeat-x;
    color: #000;
    opacity: 0.8;
    }
button.custom:hover {
    opacity: 1.0;
    }
button.custom.important {
    padding: 0.6em 1em;
    border: 1px solid transparent;
    border-color: #ffcc7f #ffcc7f hsl(36, 100%, 73%);
    border-radius: 3px;
    background-color: hsl(36, 100%, 75%);
    background-image: linear-gradient(#ffdca8, #ffcc7f);
    background-repeat: repeat-x;
    color: #222;
    opacity: 0.8;
    }
button.custom.important:hover {
    opacity: 1.0;
    }
button.custom.disabled,
button.custom[disabled] {
    border-color: #ddd #ddd hsl(36, 0%, 85%);
    background-color: hsl(36, 0%, 72%);
    background-image: linear-gradient(#f2f2f2, #dddddd);
    color: #666;
    opacity: 0.6;
    pointer-events: none;
    }
code {
    font-size: 90%;
    }
