body {
    border: 0;
    display: flex;
    flex-direction: column;
    font: 15px sans-serif;
    height: 100vh;
    margin: 0;
    overflow: hidden;
    padding: 0;
    width: 100vw;
    }
#dashboard-nav {
    border: 0;
    margin: 0;
    padding: 0;
    width: 100vw;
    z-index: 10;
    }
#dashboard-nav-widgets {
    margin: 0;
    border-bottom: 1px solid #ccc;
    padding: 4px 0 0 0;
    white-space: nowrap;
    background-color: white;
    }
#dashboard-nav-widgets span {
    padding: 0 0.5em;
    font-size: larger;
    }
.tabButton {
    margin: 0;
    border: 1px solid #ccc;
    border-top-left-radius: 3px;
    border-top-right-radius: 3px;
    padding: 4px;
    display: inline-block;
    position: relative;
    top: 1px;
    color: black;
    background-color: #eee;
    font: inherit;
    cursor: pointer;
    text-decoration: none;
    }
.tabButton:focus {
    outline: 0;
    }
.tabButton:active,.tabButton:visited {
    color: inherited;
    }
.tabButton.selected {
    border-bottom: 1px solid white;
    background-color: white;
    }
iframe {
    background-color: transparent;
    border: 0;
    flex-grow: 1;
    margin: 0;
    overflow: auto;
    padding: 0;
    width: 100%;
    }
